@@ -1,4 +1,7 @@
 #include <RcppArmadillo.h>
+using namespace Rcpp;
+
+// [[Rcpp::depends(RcppArmadillo)]]
 
 extern "C" SEXP cxxmatsub(SEXP BB, SEXP CC) {
   arma::mat B = Rcpp::as<arma::mat>(BB);
